@@ -48,8 +48,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "archiveutils.h"
 #include "dragpixmapgenerator.h"
 #include "mimetypeutils.h"
-#include "thumbnailloadjob.h"
 #include "urlutils.h"
+#include <lib/gvdebug.h>
+#include <lib/thumbnailprovider/thumbnailprovider.h>
 
 namespace Gwenview
 {
@@ -89,11 +90,13 @@ struct Thumbnail
     Thumbnail(const QPersistentModelIndex& index_, const KDateTime& mtime)
         : mIndex(index_)
         , mModificationTime(mtime)
+        , mFileSize(0)
         , mRough(true)
         , mWaitingForThumbnail(true) {}
 
     Thumbnail()
-        : mRough(true)
+        : mFileSize(0)
+        , mRough(true)
         , mWaitingForThumbnail(true) {}
 
     /**
@@ -127,6 +130,7 @@ struct Thumbnail
     void prepareForRefresh(const KDateTime& mtime)
     {
         mModificationTime = mtime;
+        mFileSize = 0;
         mGroupPix = QPixmap();
         mAdjustedPix = QPixmap();
         mFullSize = QSize();
@@ -146,8 +150,10 @@ struct Thumbnail
     /// Real size of the full image, invalid unless the thumbnail
     /// represents a raster image (not an icon)
     QSize mRealFullSize;
+    /// File size of the full image
+    KIO::filesize_t mFileSize;
     /// Whether mAdjustedPix represents has been scaled using fast or smooth
-    //transformation
+    /// transformation
     bool mRough;
     /// Set to true if mGroupPix should be replaced with a real thumbnail
     bool mWaitingForThumbnail;
@@ -172,7 +178,7 @@ struct ThumbnailViewPrivate
     QTimer mSmoothThumbnailTimer;
 
     QPixmap mWaitingThumbnail;
-    QPointer<ThumbnailLoadJob> mThumbnailLoadJob;
+    QPointer<ThumbnailProvider> mThumbnailProvider;
 
     PersistentModelIndexSet mBusyIndexSet;
     KPixmapSequence mBusySequence;
@@ -191,10 +197,10 @@ struct ThumbnailViewPrivate
                          q, SLOT(updateBusyIndexes()));
     }
 
-    void scheduleThumbnailGenerationForVisibleItems()
+    void scheduleThumbnailGeneration()
     {
-        if (mThumbnailLoadJob) {
-            mThumbnailLoadJob->removePendingItems();
+        if (mThumbnailProvider) {
+            mThumbnailProvider->removePendingItems();
         }
         mSmoothThumbnailQueue.clear();
         mScheduledThumbnailGenerationTimer.start();
@@ -210,24 +216,15 @@ struct ThumbnailViewPrivate
         QSize fullSize;
         mDocumentInfoProvider->thumbnailForDocument(url, group, &pix, &fullSize);
         mThumbnailForUrl[url] = Thumbnail(QPersistentModelIndex(index), KDateTime::currentLocalDateTime());
-        q->setThumbnail(item, pix, fullSize);
+        q->setThumbnail(item, pix, fullSize, 0);
     }
 
-    void generateThumbnailsForItems(const KFileItemList& list)
+    void appendItemsToThumbnailProvider(const KFileItemList& list)
     {
-        ThumbnailGroup::Enum group = ThumbnailGroup::fromPixelSize(mThumbnailSize.width());
-        if (!mThumbnailLoadJob) {
-            mThumbnailLoadJob = new ThumbnailLoadJob(list, group);
-            QObject::connect(mThumbnailLoadJob, SIGNAL(thumbnailLoaded(KFileItem,QPixmap,QSize)),
-                             q, SLOT(setThumbnail(KFileItem,QPixmap,QSize)));
-            QObject::connect(mThumbnailLoadJob, SIGNAL(thumbnailLoadingFailed(KFileItem)),
-                             q, SLOT(setBrokenThumbnail(KFileItem)));
-            mThumbnailLoadJob->start();
-        } else {
-            mThumbnailLoadJob->setThumbnailGroup(group);
-            Q_FOREACH(const KFileItem & item, list) {
-                mThumbnailLoadJob->appendItem(item);
-            }
+        if (mThumbnailProvider) {
+            ThumbnailGroup::Enum group = ThumbnailGroup::fromPixelSize(mThumbnailSize.width());
+            mThumbnailProvider->setThumbnailGroup(group);
+            mThumbnailProvider->appendItems(list);
         }
     }
 
@@ -290,6 +287,7 @@ ThumbnailView::ThumbnailView(QWidget* parent)
     d->mScaleMode = ScaleToFit;
     d->mThumbnailViewHelper = 0;
     d->mDocumentInfoProvider = 0;
+    d->mThumbnailProvider = 0;
     // Init to some stupid value so that the first call to setThumbnailSize()
     // is not ignored (do not use 0 in case someone try to divide by
     // mThumbnailSize...)
@@ -314,7 +312,7 @@ ThumbnailView::ThumbnailView(QWidget* parent)
     d->mScheduledThumbnailGenerationTimer.setSingleShot(true);
     d->mScheduledThumbnailGenerationTimer.setInterval(500);
     connect(&d->mScheduledThumbnailGenerationTimer, SIGNAL(timeout()),
-            SLOT(generateThumbnailsForVisibleItems()));
+            SLOT(generateThumbnailsForItems()));
 
     d->mSmoothThumbnailTimer.setSingleShot(true);
     connect(&d->mSmoothThumbnailTimer, SIGNAL(timeout()),
@@ -335,7 +333,6 @@ ThumbnailView::ThumbnailView(QWidget* parent)
 
 ThumbnailView::~ThumbnailView()
 {
-    delete d->mThumbnailLoadJob;
     delete d;
 }
 
@@ -360,6 +357,20 @@ void ThumbnailView::setModel(QAbstractItemModel* newModel)
             SIGNAL(rowsRemovedSignal(QModelIndex,int,int)));
 }
 
+void ThumbnailView::setThumbnailProvider(ThumbnailProvider* thumbnailProvider)
+{
+    GV_RETURN_IF_FAIL(d->mThumbnailProvider != thumbnailProvider);
+    if (thumbnailProvider) {
+        connect(thumbnailProvider, SIGNAL(thumbnailLoaded(KFileItem,QPixmap,QSize,qulonglong)),
+                         SLOT(setThumbnail(KFileItem,QPixmap,QSize,qulonglong)));
+        connect(thumbnailProvider, SIGNAL(thumbnailLoadingFailed(KFileItem)),
+                         SLOT(setBrokenThumbnail(KFileItem)));
+    } else {
+        disconnect(d->mThumbnailProvider, 0 , this, 0);
+    }
+    d->mThumbnailProvider = thumbnailProvider;
+}
+
 void ThumbnailView::updateThumbnailSize()
 {
     QSize value = d->mThumbnailSize;
@@ -396,7 +407,7 @@ void ThumbnailView::updateThumbnailSize()
     if (d->mScaleMode != ScaleToFit) {
         scheduleDelayedItemsLayout();
     }
-    d->scheduleThumbnailGenerationForVisibleItems();
+    d->scheduleThumbnailGeneration();
 }
 
 void ThumbnailView::setThumbnailWidth(int width)
@@ -478,8 +489,8 @@ void ThumbnailView::rowsAboutToBeRemoved(const QModelIndex& parent, int start, i
         itemList.append(item);
     }
 
-    if (d->mThumbnailLoadJob) {
-        d->mThumbnailLoadJob->removeItems(itemList);
+    if (d->mThumbnailProvider) {
+        d->mThumbnailProvider->removeItems(itemList);
     }
 
     // Update current index if it is among the deleted rows
@@ -515,6 +526,7 @@ void ThumbnailView::dataChanged(const QModelIndex& topLeft, const QModelIndex& b
         KFileItem item = fileItemForIndex(index);
         if (item.isNull()) {
             kWarning() << "Invalid item for index" << index << ". This should not happen!";
+            GV_FATAL_FAILS;
             continue;
         }
 
@@ -526,7 +538,7 @@ void ThumbnailView::dataChanged(const QModelIndex& topLeft, const QModelIndex& b
             // currently visible, and do not yet have a thumbnail for the
             // modified url.
             KDateTime mtime = item.time(KFileItem::ModificationTime);
-            if (it->mModificationTime != mtime) {
+            if (it->mModificationTime != mtime || it->mFileSize != item.size()) {
                 // dataChanged() is called when the file changes but also when
                 // the model fetched additional data such as semantic info. To
                 // avoid needless refreshes, we only trigger a refresh if the
@@ -553,7 +565,7 @@ void ThumbnailView::emitIndexActivatedIfNoModifiers(const QModelIndex& index)
     }
 }
 
-void ThumbnailView::setThumbnail(const KFileItem& item, const QPixmap& pixmap, const QSize& size)
+void ThumbnailView::setThumbnail(const KFileItem& item, const QPixmap& pixmap, const QSize& size, qulonglong fileSize)
 {
     ThumbnailForUrl::iterator it = d->mThumbnailForUrl.find(item.url());
     if (it == d->mThumbnailForUrl.end()) {
@@ -566,6 +578,7 @@ void ThumbnailView::setThumbnail(const KFileItem& item, const QPixmap& pixmap, c
     thumbnail.mFullSize = size.isValid() ? size : QSize(largeGroupSize, largeGroupSize);
     thumbnail.mRealFullSize = size;
     thumbnail.mWaitingForThumbnail = false;
+    thumbnail.mFileSize = fileSize;
 
     update(thumbnail.mIndex);
     if (d->mScaleMode != ScaleToFit) {
@@ -589,7 +602,7 @@ void ThumbnailView::setBrokenThumbnail(const KFileItem& item)
         QPixmap pix = item.pixmap(ThumbnailGroup::pixelSize(group));
         thumbnail.initAsIcon(pix);
     } else if (kind == MimeTypeUtils::KIND_DIR) {
-        // Special case for folders because ThumbnailLoadJob does not return a
+        // Special case for folders because ThumbnailProvider does not return a
         // thumbnail if there is no images
         thumbnail.mWaitingForThumbnail = false;
         return;
@@ -749,13 +762,13 @@ void ThumbnailView::keyPressEvent(QKeyEvent* event)
 void ThumbnailView::resizeEvent(QResizeEvent* event)
 {
     QListView::resizeEvent(event);
-    d->scheduleThumbnailGenerationForVisibleItems();
+    d->scheduleThumbnailGeneration();
 }
 
 void ThumbnailView::showEvent(QShowEvent* event)
 {
     QListView::showEvent(event);
-    d->scheduleThumbnailGenerationForVisibleItems();
+    d->scheduleThumbnailGeneration();
     QTimer::singleShot(0, this, SLOT(scrollToSelectedIndex()));
 }
 
@@ -793,10 +806,10 @@ void ThumbnailView::selectionChanged(const QItemSelection& selected, const QItem
 void ThumbnailView::scrollContentsBy(int dx, int dy)
 {
     QListView::scrollContentsBy(dx, dy);
-    d->scheduleThumbnailGenerationForVisibleItems();
+    d->scheduleThumbnailGeneration();
 }
 
-void ThumbnailView::generateThumbnailsForVisibleItems()
+void ThumbnailView::generateThumbnailsForItems()
 {
     if (!isVisible() || !model()) {
         return;
@@ -873,7 +886,7 @@ void ThumbnailView::generateThumbnailsForVisibleItems()
     }
 
     if (!itemMap.isEmpty()) {
-        d->generateThumbnailsForItems(itemMap.values());
+        d->appendItemsToThumbnailProvider(itemMap.values());
     }
 }
 
@@ -886,7 +899,7 @@ void ThumbnailView::updateThumbnail(const QModelIndex& index)
     } else {
         KFileItemList list;
         list << item;
-        d->generateThumbnailsForItems(list);
+        d->appendItemsToThumbnailProvider(list);
     }
 }
 
@@ -925,28 +938,22 @@ void ThumbnailView::smoothNextThumbnail()
         return;
     }
 
-    if (d->mThumbnailLoadJob) {
-        // give mThumbnailLoadJob priority over smoothing
+    if (d->mThumbnailProvider && d->mThumbnailProvider->isRunning()) {
+        // give mThumbnailProvider priority over smoothing
         d->mSmoothThumbnailTimer.start(SMOOTH_DELAY);
         return;
     }
 
     KUrl url = d->mSmoothThumbnailQueue.dequeue();
     ThumbnailForUrl::Iterator it = d->mThumbnailForUrl.find(url);
-    if (it == d->mThumbnailForUrl.end()) {
-        kWarning() <<  url << " not in mThumbnailForUrl. This should not happen!";
-        return;
-    }
+    GV_RETURN_IF_FAIL2(it != d->mThumbnailForUrl.end(), url << "not in mThumbnailForUrl.");
 
     Thumbnail& thumbnail = it.value();
     thumbnail.mAdjustedPix = d->scale(thumbnail.mGroupPix, Qt::SmoothTransformation);
     thumbnail.mRough = false;
 
-    if (thumbnail.mIndex.isValid()) {
-        update(thumbnail.mIndex);
-    } else {
-        kWarning() << "index for" << url << "is invalid. This should not happen!";
-    }
+    GV_RETURN_IF_FAIL2(thumbnail.mIndex.isValid(), "index for" << url << "is invalid.");
+    update(thumbnail.mIndex);
 
     if (!d->mSmoothThumbnailQueue.isEmpty()) {
         d->mSmoothThumbnailTimer.start(0);
@@ -960,13 +967,13 @@ void ThumbnailView::reloadThumbnail(const QModelIndex& index)
         kWarning() << "Invalid url for index" << index;
         return;
     }
-    ThumbnailLoadJob::deleteImageThumbnail(url);
+    ThumbnailProvider::deleteImageThumbnail(url);
     ThumbnailForUrl::Iterator it = d->mThumbnailForUrl.find(url);
     if (it == d->mThumbnailForUrl.end()) {
         return;
     }
     d->mThumbnailForUrl.erase(it);
-    generateThumbnailsForVisibleItems();
+    generateThumbnailsForItems();
 }
 
 void ThumbnailView::setCreateThumbnailsForRemoteUrls(bool createRemoteThumbs)
